package de.hendriklipka.config.impl;

import de.hendriklipka.config.impl.BlockImpl;
import de.hendriklipka.config.Value;
import de.hendriklipka.config.ConfigFileException;
import de.hendriklipka.config.Block;
import de.hendriklipka.config.ReplacementHandler;

import java.util.Enumeration;

public final class ValueImpl implements Value
{
    protected String _value=null;
    protected BlockImpl _parent=null;
    protected String _id=null;

    protected boolean _replacementHandled=false;

    public ValueImpl(String value, BlockImpl parent)
    {
        _value=value;
        _parent=parent;
    }

    public String getValue() throws ConfigFileException
    {
        if (!_replacementHandled)
        {
            doReplacement(_parent);
            _replacementHandled=true;
        }
        return _value;
    }

    private int needsReplacement(int offset)
    {
        while (true)
        {
            // do we have the start of a replacement string?
            int pos=_value.indexOf("${", offset);
            if (-1==pos)
                return -1;
            // is it escaped?
            if (pos>0&&('\\'==_value.charAt(pos-1)))
            {
                // count \ characters backwards
                // we do this to determine if the \ we just found itself was escaped or not
                int count=getEscapeCount(pos-2);
                // no, its not escaped, so the $ is escaped, so start again at next position
                if (1==(count%2))
                {
                    offset=pos+2;
                    continue;
                }
            }
            // look for the final character of the replacement
            int pos2=_value.indexOf('}', pos+2);
            if (-1==pos2)
                return -1;
            return pos+2;
        }
    }

    private int getEscapeCount(int i)
    {
        int count=1;
        while (i>=0)
        {
            if ('\\'==_value.charAt(i))
                count++;
            else
                break;
            i--;
        }
        return count;
    }

    private synchronized boolean doReplacement(Block root) throws ConfigFileException
    {
        StringBuffer newValue=new StringBuffer();
        int offset=0;
        while (true)
        {
            int pos=needsReplacement(offset);
            if (-1==pos)
                break;
            int pos2=_value.indexOf('}', pos+1);
            String replaceName=_value.substring(pos, pos2);
            String replaceValue=determineReplacementValue(replaceName); // throws exception if nothing found

            // transfer string before replacement into result
            String start=_value.substring(offset, pos-2);
            start=replaceEscapeStrings(start);
            newValue.append(start);
            // append replacement value
            newValue.append(replaceValue);
            // set offset behind original replacement
            offset=pos2+1;
        }
        // if something is left, transfer it also
        if (offset<_value.length())
        {
            String left=_value.substring(offset);
            left=replaceEscapeStrings(left);
            newValue.append(left);
        }
        _value=newValue.toString();
        return true;
    }

    private String determineReplacementValue(String replaceName) throws ConfigFileException
    {
        String replaceValue=null;
        Enumeration handlers=ConfigurationImpl.getInstance().getReplacementHandlers();
        while (handlers.hasMoreElements())
        {
            ReplacementHandler handler=(ReplacementHandler) handlers.nextElement();
            replaceValue=handler.determineReplacementValue(replaceName, _parent);
            if (null!=replaceValue)
                break;
        }
        if (null==replaceValue)
        {
            throw new ConfigFileException("got no replacement block for "+replaceName+" in "+_parent._name);
        }
        return replaceValue;
    }

    private String replaceEscapeStrings(String value) throws ConfigFileException
    {
        StringBuffer result=new StringBuffer(value.length());
        if (0!=value.length())
        {
            char chars[]=new char[value.length()];
            value.getChars(0, value.length(), chars, 0);
            for (int i=0; i<chars.length; i++)
            {
                char c=chars[i];
                if (('\\'==c)&&(i!=(chars.length-1)))
                {
                    i++;
                    c=chars[i];
                    if (c!='\\'&&c!='$')
                    {
                        throw new ConfigFileException("replacement error! char "+c+" cannot be escaped in value "+_value);
                    }
                }
                result.append(c);
            }
        }
        return result.toString();
    }
    
    public String toString()
    {
        return _value;
    }
}
