package de.hendriklipka.databus;

import java.util.Hashtable;
import java.util.NoSuchElementException;
import java.util.Enumeration;

public class ConfigElement
{
    protected String _data=null;
    protected Hashtable _childs=new Hashtable();
    protected String _name=null;

    ConfigElement(String name, String data)
    {
        _name=name;
        _data=data;
    }

    public String getName()
    {
        return _name;
    }

    public String getData()
    {
        return _data;
    }

    public ConfigElement getChild(String name)
    {
        return (ConfigElement)_childs.get(name);
    }

    public ConfigElement[] getChilds()
    {
        ConfigElement[] cea=new ConfigElement[_childs.size()];
        int i=0;
        Enumeration e=_childs.keys();
        while (e.hasMoreElements())
        {
            String key=(String)e.nextElement();
            ConfigElement ce=(ConfigElement)_childs.get(key);
            cea[i]=ce;
        }
        return cea;
    }

    public String getValue(String name) throws NoSuchElementException
    {
        int pos=name.indexOf('.');
        if (-1==pos)
        {
            ConfigElement ce=getChild(name);
            if (null==ce)
            {
                throw new NoSuchElementException(name);
            }
            return ce.getData();
        }
        String s1=name.substring(0,pos);
        ConfigElement ce=getChild(s1);
        if (null==ce)
        {
            throw new NoSuchElementException(s1);
        }
        String s2=name.substring(pos+1);
        return ce.getValue(s2); // start recursion
    }

    public boolean existValue(String name)
    {
        int pos=name.indexOf('.');
        if (-1==pos)
        {
            ConfigElement ce=getChild(name);
            return (null!=ce);
        }
        String s1=name.substring(0,pos);
        ConfigElement ce=getChild(s1);
        return (null!=ce);
    }

    public int getDataAsInt(String name) throws NoSuchElementException
    {
        String s=getValue(name);
        try
        {
            return Integer.parseInt(s);
        }
        catch(NumberFormatException ex)
        {
            throw new NoSuchElementException(ex.getMessage());
        }
    }

    public long getDataAsLong(String name) throws NoSuchElementException
    {
        String s=getValue(name);
        try
        {
            return Long.parseLong(s);
        }
        catch(NumberFormatException ex)
        {
            throw new NoSuchElementException(ex.getMessage());
        }
    }

    public boolean getDataAsBoolean(String name) throws NoSuchElementException
    {
        String s=getValue(name).toLowerCase();
        if (s.equals("yes")||s.equals("true")||s.equals("1"))
        {
            return true;
        }
        if (s.equals("no")||s.equals("false")||s.equals("0"))
        {
            return false;
        }
        throw new NoSuchElementException("invalid value for <"+name+">="+s);
    }

    public double getDataAsDouble(String name) throws NoSuchElementException
    {
        String s=getValue(name);
        try
        {
            return Double.parseDouble(s);
        }
        catch(NumberFormatException ex)
        {
            throw new NoSuchElementException(ex.getMessage());
        }
    }

    void addChild(String name, ConfigElement element)
    {
        _childs.put(name,element);
    }

    void clearAllElements()
    {
        _childs.clear();
    }
}